import click

from kecpkg.commands.build import build
from kecpkg.commands.new import new
from kecpkg.commands.prune import prune
from kecpkg.commands.purge import purge
from kecpkg.commands.upload import upload
from kecpkg.commands.config import config
from kecpkg.commands.utils import CONTEXT_SETTINGS


class AliasedGroup(click.Group):
    """Intermediate class to combine the kecpkg command groups."""

    pass


@click.group(cls=AliasedGroup, context_settings=CONTEXT_SETTINGS)
@click.version_option()
def kecpkg():
    """KE-chain Package manager toolset."""
    pass


kecpkg.add_command(new)
kecpkg.add_command(build)
kecpkg.add_command(upload)
kecpkg.add_command(purge)
kecpkg.add_command(prune)
kecpkg.add_command(config)



import json
import os
from collections import OrderedDict
from copy import deepcopy

import sys
from atomicwrites import atomic_write

from kecpkg.commands.utils import echo_failure
from kecpkg.utils import ensure_dir_exists, create_file, get_package_dir

SETTINGS_FILENAME = '.kecpkg_settings.json'
SETTINGS_FILE = os.path.join(os.getcwd(), SETTINGS_FILENAME)

DEFAULT_SETTINGS = OrderedDict([
    ('version', '0.0.1'),
    ('pyversions', ['2.7', '3.5']),
    ('python_version', '3.5'),
    ('venv_dir', 'venv'),
    ('entrypoint_script', 'script'),
    ('entrypoint_func', 'main'),
    ('build_dir', 'dist'),
    ('requirements_filename', 'requirements.txt')
])

EXCLUDE_DIRS_IN_BUILD = [
    'venv', 'dist', '.idea', '.tox', '.cache', '.git', 'venv*', '.env', '__pycache__', 'develop-eggs', 'downloads',
    'eggs', 'lib', 'lib64', 'sdist', 'wheels', '.hypothesis', '.ipynb_checkpoints', '.mypy_cache'
]

EXCLUDE_PATHS_IN_BUILD = [
    '.gitignore', '*.pyc', '*.pyo', '*.pyd', '*$py.class', '*.egg-info', '.installed.cfg', '.coveragerc', '*.egg',
    'pip-log.txt', '*.log', 'pip-delete-this-directory.txt', '.coverage*', 'nosetests.xml', 'coverage.xml', '*.cover',
    'env.bak', 'venv.bak', 'pip-selfcheck.json', '*.so'
]

EXCLUDE_IN_BUILD = EXCLUDE_DIRS_IN_BUILD + EXCLUDE_PATHS_IN_BUILD


def copy_default_settings():
    """Copy the default settings to a new dict."""
    return deepcopy(DEFAULT_SETTINGS)


def load_settings(lazy=False, package_dir=None):
    """
    Load settings from disk.

    :param lazy: (optional) does lazy loading (default to False)
    :param package_dir: (optional) loads the settings from a package dir
    :return: settings dictionary
    """
    settings_filepath = get_settings_filepath(package_dir)
    if lazy and not os.path.exists(settings_filepath):
        return {}
    elif not os.path.exists(settings_filepath):
        echo_failure('Could not find a settingsfile in path: {}'.format(settings_filepath))
        sys.exit(404)
    else:
        with open(settings_filepath, 'r') as f:
            return json.loads(f.read(), object_pairs_hook=OrderedDict)


def get_settings_filepath(package_dir=None):
    """
    Return the filepath of the settings file.

    :param package_dir: (optional) Package dir to search in
    :return: path tot the settings file
    """
    if package_dir:
        return os.path.join(package_dir, SETTINGS_FILENAME)
    else:
        return SETTINGS_FILE


def save_settings(settings, package_dir=None):
    """
    Save settings in path (in the package).

    :param settings: settings to save
    :param package_dir: (optional) package_dir to save to
    :return: None
    """
    if settings.get('package_name') and not package_dir:
        package_dir = get_package_dir(settings.get('package_name'))
    settings_filepath = get_settings_filepath(package_dir)

    ensure_dir_exists(os.path.dirname(settings_filepath))
    with atomic_write(settings_filepath, overwrite=True) as f:
        f.write(json.dumps(settings, indent=4))


def restore_settings(package_dir=None):
    """
    Restore settings to their defaults (overwrite old settings).

    :param package_dir: (optional) package dir to search the settings file in
    :return: None
    """
    settings_filepath = get_settings_filepath(package_dir)

    create_file(settings_filepath)
    settings = copy_default_settings()
    if package_dir:
        package_name = os.path.dirname(package_dir)
        settings['package_name'] = package_name
    save_settings(settings=settings)



import os

from click.testing import CliRunner

from kecpkg.cli import kecpkg
from kecpkg.utils import get_package_dir
from tests.utils import temp_chdir, BaseTestCase


class TestCommandPurge(BaseTestCase):
    def test_purge_non_interactive(self):
        pkgname = 'new_pkg'

        with temp_chdir() as d:
            runner = CliRunner()
            result = runner.invoke(kecpkg, ['new', pkgname, '--no-venv'])
            package_dir = get_package_dir(pkgname)
            self.assertTrue(os.path.exists(package_dir))

            result = runner.invoke(kecpkg, ['purge', pkgname, '--force'])
            self.assertFalse(os.path.exists(package_dir))





import os
import socket
from contextlib import contextmanager

from unittest import TestCase

import pytest
import six
from click.testing import CliRunner


class BaseTestCase(TestCase):
    def SetUp(self):
        self.runner = CliRunner()

    def assertExists(self, path):
        self.assertTrue(os.path.exists(path), "Path `{}` does not exists".format(path))

@contextmanager
def temp_chdir(cwd=None):
    if six.PY3:
        from tempfile import TemporaryDirectory
        with TemporaryDirectory() as tempwd:
            origin = cwd or os.getcwd()
            os.chdir(tempwd)

            try:
                yield tempwd if os.path.exists(tempwd) else ''
            finally:
                os.chdir(origin)
    else:
        from tempfile import mkdtemp
        tempwd = mkdtemp()
        origin=cwd or os.getcwd()
        os.chdir(tempwd)
        try:
            yield tempwd if os.path.exists(tempwd) else ''
        finally:
            os.chdir(origin)


def connected_to_internet():  # no cov
    if os.environ.get('CI') and os.environ.get('TRAVIS'):
        return True
    try:
        # Test availability of DNS first
        host = socket.gethostbyname('www.google.com')
        # Test connection
        socket.create_connection((host, 80), 2)
        return True
    except:
        return False

def touch_file(path):
    """Create an empty file in path.

    :param path: path (filename)
    """
    with open(path, 'a'):
        os.utime(path, None)



requires_internet = pytest.mark.skipif(
    not connected_to_internet(), reason='Not connected to internet'
)


"""
Part of the kecpkg-tools project.

Parts are borrowed from hatch Those parts are are released under the MIT license
"""
import fnmatch
import os
import platform
import re
import shutil
import sys
from contextlib import contextmanager

import six

from kecpkg.commands.utils import echo_failure, echo_info, echo_warning


def ensure_dir_exists(d):
    """Ensure that directory exists, otherwise make directory."""
    if not os.path.exists(d):
        os.makedirs(d)


def create_file(filepath, content=None, overwrite=True):
    """
    Create file and optionally fill it with content.

    Will overwrite file already in place if overwrite flag is set

    :param filepath: full path to a file to create
    :param content:
    :return:
    """
    ensure_dir_exists(os.path.dirname(os.path.abspath(filepath)))
    # if overwrite is set to True overwrite file, otherwise if file exist, exit.

    if not os.path.exists(filepath) or (os.path.exists(filepath) and overwrite):
        with open(filepath, 'w') as fd:
            # os.utime(filepath, times=None)
            if content:
                fd.write(content)
    else:
        echo_failure("File '{}' already exists.".format(filepath))
        sys.exit(1)


def copy_path(sourcepath, destpath):
    """
    Copy path.

    :param sourcepath: source path to copy from, if dir, copy subtree
    :param destpath: destination path to copy to
    """
    if os.path.isdir(sourcepath):
        if six.PY3:
            shutil.copytree(sourcepath, os.path.join(destpath, basepath(sourcepath)),
                            copy_function=shutil.copy)
        else:
            shutil.copytree(sourcepath, os.path.join(destpath, basepath(sourcepath)))

    else:
        shutil.copy(sourcepath, destpath)


def remove_path(path):
    """
    Remove directory structure.

    :param path: path to remove
    """
    try:
        shutil.rmtree(path)
    except (IOError, OSError):
        try:
            os.remove(path)
        except (IOError):
            pass


def basepath(path):
    """Get full basepath from path."""
    return os.path.basename(os.path.normpath(path))


def normalise_name(raw_name):
    """
    Normalise the name to be used in python package allowable names.

    conforms to PEP-423 package naming conventions

    :param raw_name: raw string
    :return: normalised string
    """
    return re.sub(r"[-_. ]+", "_", raw_name).lower()


def get_package_dir(package_name=None, fail=True):
    """
    Check and retrieve the package directory.

    :param package_name: (optional) package name
    :param fail: (optional, default=True) fail hard with exit when no package dir found
    :return: full path name to the package directory
    """
    from kecpkg.settings import SETTINGS_FILENAME

    def _inner(d):
        if os.path.exists(os.path.join(d, SETTINGS_FILENAME)):
            return d
        elif os.path.exists(os.path.join(d, 'package_info.json')):
            return d
        else:
            return None

    package_dir = _inner(os.getcwd())
    if not package_dir:
        package_dir = _inner(os.path.join(os.getcwd(), package_name))
    if not package_dir:
        package_dir = _inner(package_name)
    if not package_dir:
        echo_failure('This does not seem to be a package in path `{}` - please check that there is a '
                     '`package_info.json` or a `{}`'.format(package_dir, SETTINGS_FILENAME))
        if fail:
            sys.exit(1)
    else:
        return package_dir


def get_package_name():
    """
    Provide the name of the package (in current dir).

    :param fail: ensure that directory search does not fail in a exit.
    :return: package name or None
    """
    package_dir = get_package_dir(fail=False)
    if package_dir:
        return os.path.basename(package_dir)
    else:
        return None


def get_artifacts_on_disk(root_path, additional_exclude_paths=None, default_exclude_paths=None, verbose=False):
    """
    Retrieve all artifacts on disk.

    :param root_path: root_path to collect all artifacts from
    :param exclude_paths: (optional) directory names and filenames to exclude
    :return: dictionary with {'property_id': ['attachment_path1', ...], ...}
    """
    from kecpkg.settings import EXCLUDE_IN_BUILD
    exclude_paths = default_exclude_paths or EXCLUDE_IN_BUILD
    if verbose:
        echo_info("basic excluded paths are: `{}`".format(exclude_paths))

    # get additional exclude paths from the settings file
    if additional_exclude_paths and isinstance(additional_exclude_paths, list):
        exclude_paths.extend(additional_exclude_paths)
        if verbose:
            echo_info("additional exclude paths are: `{}`".format(additional_exclude_paths))

    if not os.path.exists(root_path):
        echo_failure("The root path: '{}' does not exist".format(root_path))
        sys.exit(1)

    # getting all attachments
    artifacts = []
    for root, dirs, filenames in os.walk(root_path, topdown=True):
        for exclude_path in exclude_paths:
            if exclude_path in dirs:
                dirs.remove(exclude_path)
                if verbose:
                    echo_warning("Ignored path `{}`".format(exclude_path))

        for filename in filenames:
            # print([(filename,ptrn,fnmatch.fnmatch(filename, ptrn)) for ptrn in exclude_paths])
            if not any([fnmatch.fnmatch(filename, ptrn) for ptrn in exclude_paths]):
                full_artifact_subpath = '{}{}{}'.format(root, os.path.sep, filename). \
                    replace('{}{}'.format(root_path, os.path.sep), '')
                artifacts.append(full_artifact_subpath)
                if verbose:
                    echo_info('Found `{}`'.format(full_artifact_subpath))
            else:
                if verbose:
                    echo_warning('Ignored `{}`'.format(filename))

    if verbose:
        echo_info('{}'.format(artifacts))
    return artifacts


# Python Operation regarding Virtual environments
# Graceously borrowed From hatch package.

__platform = platform.system()
ON_MACOS = os.name == 'mac' or __platform == 'Darwin'
ON_WINDOWS = NEED_SUBPROCESS_SHELL = os.name == 'nt' or __platform == 'Windows'
VENV_FLAGS = {
    '_HATCHING_',
    'VIRTUAL_ENV',
    'CONDA_PREFIX',
}


def venv_ignored():
    """
    Check if virtual env is to be ignored.

    Graceously borrowed From hatch package.
    """
    return os.environ.get('_IGNORE_VENV_') == '1'


def venv_active():
    """
    Check if virtual env is active.

    Graceously borrowed From hatch package.
    """
    return bool(VENV_FLAGS & set(os.environ)) and not venv_ignored()


def get_proper_python():  # no cov
    """
    Retrieve the proper python version on the platform.

    Graceously borrowed From hatch package.
    """
    if not venv_active():
        default_python = os.environ.get('_DEFAULT_PYTHON_', None)
        if default_python:
            return default_python
        elif not ON_WINDOWS:
            return 'python3'
    return 'python'


def get_proper_pip():  # no cov
    """
    Retrieve the propery pip executable on the platform.

    Graceously borrowed From hatch package.
    """
    if not venv_active():
        default_pip = os.environ.get('_DEFAULT_PIP_', None)
        if default_pip:
            return default_pip
        elif not ON_WINDOWS:
            return 'pip3'
    return 'pip'


def locate_exe_dir(d, check=True):
    """
    Locate the python or pip executables on the platform.

    Graceously borrowed From hatch package.
    """
    exe_dir = os.path.join(d, 'Scripts') if ON_WINDOWS else os.path.join(d, 'bin')
    if check and not os.path.isdir(exe_dir):
        raise OSError('Unable to locate python virtual environment executables directory.')
    return exe_dir


@contextmanager
def env_vars(evars, ignore=None):
    """
    Contextmanager to provide filtering on the environment variables already on the system.

    Graceously borrowed From hatch package.

    :param evars: new environment variables to inject or override
    :param ignore: ignored environment variables
    :return: context in which the environment variable dictionary is rewritten
    """
    ignore = ignore or {}
    ignored_evars = {}
    old_evars = {}

    for ev in evars:
        if ev in os.environ:
            old_evars[ev] = os.environ[ev]
        os.environ[ev] = evars[ev]

    for ev in ignore:
        if ev in os.environ:  # no cov
            ignored_evars[ev] = os.environ[ev]
            os.environ.pop(ev)

    try:
        yield
    finally:
        for ev in evars:
            if ev in old_evars:
                os.environ[ev] = old_evars[ev]
            else:
                os.environ.pop(ev)

        for ev in ignored_evars:
            os.environ[ev] = ignored_evars[ev]


@contextmanager
def venv(venv_path, evars=None):
    """
    Operate within the confines of a virtual environment.

    Graceously borrowed From hatch package.

    :param venv_path: virtual environment path
    :param evars: additional environment variables, which will be overwritten
    :return: context in which the virtual environment directory is set to the venv_path.
    """
    venv_exe_dir = locate_exe_dir(venv_path)

    evars = evars or {}
    evars['_HATCHING_'] = '1'
    evars['VIRTUAL_ENV'] = venv_path
    evars['PATH'] = '{}{}{}'.format(
        venv_exe_dir, os.pathsep, os.environ.get('PATH', '')
    )

    with env_vars(evars, ignore={'__PYVENV_LAUNCHER__'}):
        yield venv_exe_dir



import os

import click

from kecpkg.commands.utils import CONTEXT_SETTINGS, echo_warning, echo_failure, echo_success
from kecpkg.utils import remove_path, get_package_dir


@click.command(context_settings=CONTEXT_SETTINGS,
               short_help="Purge and delete a project (No reverse)")
@click.argument('package', required=False)
@click.option('--force', '-f', is_flag=True, help="Force purge (no confirmation)")
def purge(package, **options):
    """
    Purge and clean a package directory structure.

    :param package: Name of the kecpkg package
    :param options:
    :return:
    """
    package_name = package or click.prompt('Provide package name')
    package_dir = get_package_dir(package_name)

    if os.path.exists(package_dir):
        if options.get('force') or click.confirm(
                "Do you want to purge and completely remove '{}'?".format(package_name)):
            remove_path(package_dir)
            if not os.path.exists(package_dir):
                echo_success('Package `{}` is purged and removed from disk'.format(package_name))
            else:
                echo_failure('Something went wrong pruning pacakage `{}`'.format(package_name))
        else:
            echo_warning('Package `{}` will not be purged'.format(package_name))
    else:
        echo_failure('Package `{}` does not exist'.format(package_name))



import os
import sys

import click as click
from pykechain import Client, get_project

from kecpkg.commands.utils import CONTEXT_SETTINGS, echo_info, echo_success, echo_failure
from kecpkg.settings import load_settings, save_settings
from kecpkg.utils import get_package_dir, get_package_name


@click.command(context_settings=CONTEXT_SETTINGS,
               short_help="Upload package to a KE-chain 2 scope")
@click.argument('package', required=False)
@click.option('--url', '-U', help="URL of the KE-chain instance (eg. https://<instance>.ke-chain.com)")
@click.option('--username', '-u', help="username for KE-chain", default=os.environ.get('USER', ''))
@click.option('--password', '-p', help="password for KE-chain")
@click.option('--token', help="token for KE-chain access")
@click.option('--scope', help="scope name to upload the kecpkg to")
@click.option('--scope-id', 'scope_id', help="UUID of the scope to upload the kecpkg to", type=click.UUID)
@click.option('--service-id', 'service_id', help="(optional) id of the service to reupload", type=click.UUID)
@click.option('--reupload', '--replace', '-r', is_flag=True, default=False,
              help="(optional) reupload the kecpkg to an already existing service")
@click.option('--interactive', '-i', is_flag=True, help="interactive mode; guide me through the upload")
@click.option('--kecpkg', help="(optional) path to the kecpkg file to upload", type=click.Path())
@click.option('--store/--no-store', is_flag=True, default=True,
              help="(optional) flag to store provided interactive information to settings (except pass)")
def upload(package=None, url=None, username=None, password=None, token=None, scope=None, scope_id=None, kecpkg=None,
           **options):
    """
    Upload built kecpkg to KE-chain.

    If no options are provided, the interactive mode is triggered.
    """
    package_name = package or get_package_name() or click.prompt('Package name')
    settings = load_settings(package_dir=get_package_dir(package_name))

    if not url or not ((username and password) or token):
        url = click.prompt('Url (incl http(s)://)', default=settings.get('url') or url)
        username = click.prompt('Username', default=settings.get('username') or username)
        password = click.prompt('Password', hide_input=True)
        # set the interactive world to True for continuation sake
        options['interactive'] = True
    elif not options.get('interactive'):
        url = url or settings.get('url')
        username = username or settings.get('username')
        token = token or settings.get('token')
        scope_id = scope_id or settings.get('scope_id')

    client = Client(url)
    client.login(username=username, password=password, token=token)

    # scope finder
    if not scope_id and settings.get('scope_id') and \
            click.confirm("Do you wish to use the stored `scope_id` in settings: `{}`".format(
                settings.get('scope_id')), default=True):
        scope_id = settings.get('scope_id')

    if not scope_id:
        scopes = client.scopes()
        scope_matcher = [dict(number=i, scope_id=scope.id, scope=scope.name) for i, scope in
                         zip(range(1, len(scopes)), scopes)]

        # nice UI
        echo_info('Choose from following scopes:')
        for match_dict in scope_matcher:
            echo_info("{number} | {scope_id:.8} | {scope}".format(**match_dict))

        scope_match = None
        while not scope_match:
            scope_guess = click.prompt('Row number, part of Id or Scope')
            scope_match = validate_scopes(scope_guess, scope_matcher)

        echo_success("Scope selected: '{scope}' ({scope_id})".format(**scope_match))
        scope_id = scope_match['scope_id']

    scope_to_upload = get_project(url, username, password, token, scope_id=scope_id)

    # service reupload
    service_id = options.get('service_id') or settings.get('service_id')
    if options.get('reupload') and not service_id:
        echo_failure('Please provide a service id to reupload to.')
    elif service_id and not options.get('reupload') and options.get('interactive'):
        if click.confirm("Do you wish to *replace* the previously uploaded service: `{}`".format(service_id),
                         default=True):
            service_id = service_id
        else:
            service_id = None

    # store to settings
    if options.get('store'):
        settings.update(dict(
            url=url,
            username=username,
            scope_id=str(scope_id)
        ))
        if service_id:
            settings['service_id'] = str(service_id)
        save_settings(settings)

    # do upload
    build_path = os.path.join(get_package_dir(package_name), settings.get('build_dir'))
    if not os.path.exists(build_path):
        echo_failure('Cannot find build path, please do `kecpkg build` first')
        sys.exit(400)

    upload_package(scope_to_upload, build_path, kecpkg, service_id=service_id, settings=settings,
                   store_settings=options.get('store'))


def upload_package(scope, build_path=None, kecpkg_path=None, service_id=None, settings=None, store_settings=True):
    """
    Upload the package from build_path to the right scope, create a new KE-chain SIM service.

    :param scope: Scope object (pykechain)
    :param build_path: path to the build directory in which the to-be uploaded script resides
    :param kecpkg_path: path to the kecpkg file to upload (no need to provide build_path)
    :param service_id: UUID of the service to upload to
    :param settings: settings of the package
    :param store_settings: store the settings after update (eg service_id after upload)
    :return: None
    """
    # if not (kecpkg_path and not build_path) or not (build_path and not kecpkg_path):
    #     echo_failure("You should provide a build path or a kecpkg path")
    #     sys.exit(404)
    if kecpkg_path and os.path.exists(kecpkg_path):
        kecpkg_path = kecpkg_path
    else:

        built_kecpkgs = os.listdir(build_path)
        if not kecpkg_path and len(built_kecpkgs) > 1 and settings.get('version'):
            built_kecpkgs = [f for f in built_kecpkgs if settings.get('version') in f]
        if not kecpkg_path and len(built_kecpkgs) == 1:
            kecpkg_path = os.path.join(build_path, built_kecpkgs[0])
        else:
            echo_info('Provide correct filename to upload')
            echo_info('\n'.join(os.listdir(build_path)))
            kecpkg_filename = click.prompt('Filename')
            kecpkg_path = os.path.join(build_path, kecpkg_filename)

    if kecpkg_path and os.path.exists(kecpkg_path):
        # ready to upload
        echo_info('Ready to upload `{}`'.format(os.path.basename(kecpkg_path)))
    else:
        echo_failure('Unable to locate kecpkg to upload')
        sys.exit(404)

    # get meta and prepare 2 stage submission
    # 1. fill service information
    # 2. do upload

    if service_id:
        service = scope.service(pk=service_id)
        service.upload(kecpkg_path)
        service.edit(
            name=settings.get('package_name'),
            description=settings.get('description', ''),
            script_version=settings.get('version', '')
        )
    else:
        # Create new service in KE-chain
        service = scope.create_service(
            name=settings.get('package_name'),
            description=settings.get('description', ''),
            version=settings.get('version', ''),
            service_type='PYTHON SCRIPT',
            environment_version=settings.get('python_version'),
            pkg_path=kecpkg_path
        )

    # Wrap up party!
    echo_success("kecpkg `{}` successfully uploaded to KE-chain.".format(os.path.basename(kecpkg_path)))
    success_url = "{api_root}/#scopes/{scope_id}/scripts/{service_id}".format(
        api_root=scope._client.api_root,
        scope_id=scope.id,
        service_id=service.id
    )
    echo_success("To view the newly created service, go to: `{}`".format(success_url))

    # update settings
    if store_settings:
        settings['service_id'] = str(service.id)
        from datetime import datetime
        settings['last_upload'] = str(datetime.now().isoformat())
        save_settings(settings)


def validate_scopes(scope_guess, scope_matcher):
    """Check the scope guess against a set of possible scopes and return correct scope_id."""
    scope_matches = []
    for scope_match in scope_matcher:
        # order is important as '1' can also be in UUID and Name, so we use exclusive if statements
        if scope_guess == str(scope_match['number']):
            scope_matches.append(scope_match)
        elif len(scope_guess) >= 2 and scope_guess.lower() in scope_match['scope_id'].lower():
            scope_matches.append(scope_match)
        elif scope_guess.lower() in scope_match['scope'].lower():
            scope_matches.append(scope_match)

    # only return when a single scope is matched
    if len(scope_matches) == 1:
        return scope_matches[0]
    print('Could not match; be more specific')
    return None



import os

import click

from kecpkg.commands.utils import CONTEXT_SETTINGS, echo_info, echo_success
from kecpkg.settings import load_settings, copy_default_settings, save_settings, SETTINGS_FILENAME
from kecpkg.utils import get_package_dir, copy_path


@click.command(context_settings=CONTEXT_SETTINGS,
               short_help="Finds and updated the configuration of the kecpkg")
@click.argument('package', required=False)
@click.option('--init', is_flag=True, help="will init a settingsfile if not found")
@click.option('--interactive', '-i', is_flag=True, help="interactive mode; guide me through the settings")
@click.option('--verbose', '-v', is_flag=True, help="be more verbose (print settings)")
def config(package, **options):
    r"""Manage the configuration (or settings) of the package.

    The various settings in the .kecpkg-settings.json file are:

    \b
    package_name:   name of the package
    version:        version number of the package
    description:    longer description of the package
    name:           name of the author of the package
    email:          email of the author of the package
    python_version: python version on which the kecpkg will run, corresponds
                    with an executable KE-crunch environment
    entrypoint_script: the name of the script that will be executed first
    entrypoint_func: function name inside the script that will be executed.
                    Ensure that it takes *args, **kwargs.
    venv_dir:       python virtual environment directory in the development environment
    requirements_filename: name of the requirements file with list of package that
                    will be installed before running
    build_dir:      directory where the built kecpkg will be stored
    exclude_paths:  list of paths that will be excluded from the package, next to
                    the build in excludes
    url:            url where the package will be uploaded
    token:          token of the user under which the package is uploaded
    scope_id:       identification of the scope under which the package is uploaded
    service_id:     identification under which the package is re-uploaded
                    (or recently uploaded)
    last_upload:    date and time of the last upload
    """
    echo_info('Locating package ``'.format(package))
    package_dir = get_package_dir(package_name=package)
    package_name = os.path.basename(package_dir)
    echo_info('Package `{}` has been selected'.format(package_name))

    if options.get('init'):
        if os.path.exists(os.path.join(package_dir, SETTINGS_FILENAME)) and \
                click.confirm('Are you sure you want to overwrite the current settingsfile '
                              '(old settings will be a backup)?'):
            copy_path(os.path.join(package_dir, SETTINGS_FILENAME),
                      os.path.join(package_dir, "{}-backup".format(SETTINGS_FILENAME)))
        echo_info('Creating new settingsfile')
        settings = copy_default_settings()
        settings['package_name'] = package_name
        save_settings(settings, package_dir=package_dir)

    settings = load_settings(package_dir=package_dir)
    if options.get('interactive'):
        settings['version'] = click.prompt('Version', default=settings.get('version', '0.0.1'))
        settings['description'] = click.prompt('Description', default=settings.get('description', ''))
        settings['name'] = click.prompt('Author', default=settings.get('name', os.environ.get('USER', '')))
        settings['email'] = click.prompt('Author\'s email', default=settings.get('email', ''))
        settings['python_version'] = click.prompt('Python version (choose from: {})'.
                                                  format(settings.get('pyversions')), default='3.5')
        settings['exclude_paths'] = click.prompt("Exclude additional paths from kecpkg (eg. 'data, input')",
                                                 default=settings.get('exclude_paths', ''),
                                                 value_proc=process_additional_exclude_paths)
        save_settings(settings, package_dir=package_dir)

    if options.get('verbose'):
        for k, v in settings.items():
            echo_info("  {}: '{}'".format(k, v))

    if not options.get('interactive'):
        echo_success('Settings file identified and correct')


def process_additional_exclude_paths(raw_value):
    """Process additional list of exclude paths and return a list."""
    assert isinstance(raw_value, str), "The value should be a string, got: {}".format(type(raw_value))

    pathlist = []
    raw_pathlist = raw_value.split(',')
    for raw_path in raw_pathlist:
        pathlist.append(raw_path.strip())
    return pathlist



import os

from jinja2 import Environment, FileSystemLoader

from kecpkg.utils import create_file

__environment = None


def get_environment():
    """Retrieve the jinja 2 environment as this is a singleton."""
    global __environment
    if not __environment:
        __environment = Environment(loader=FileSystemLoader(os.path.join(os.path.dirname(__file__), 'templates')))
    return __environment


def get_template(template):
    """
    From the rendering environment, retrieve the template.

    :param template: template name
    :return: Template
    """
    return get_environment().get_template(template)


def render_to_template(template, content):
    """
    Render content (dict) to the template.

    :param template: name of the jinja2 template file
    :param content: dictionary with content to render
    :return: rendered template (as string)
    """
    return get_template(template).render(content)


def render_to_file(filename, content, target_dir=None, template=None):
    """
    Render content to a template file in a target_dir.

    :param filename: filename (basename) to render to
    :param content: dictionary to render in the template
    :param target_dir: (optional) subdirectory or fullpath where to create the file
    :param template: (optional) template name (if not provided will be `<filename>.template`
    :return: None
    """
    # alias for render_to_template
    template = template or '{}.template'.format(filename)
    target_dir = target_dir or os.getcwd()
    filepath = os.path.join(target_dir, filename)

    filecontents = render_to_template(template=template, content=content)
    create_file(filepath=filepath, content=filecontents)



import os
from zipfile import ZipFile

import click as click

from kecpkg.commands.utils import CONTEXT_SETTINGS, echo_info
from kecpkg.settings import load_settings
from kecpkg.utils import ensure_dir_exists, remove_path, get_package_dir, get_artifacts_on_disk


@click.command(context_settings=CONTEXT_SETTINGS,
               short_help="Build the package and create a kecpkg file")
@click.argument('package', required=False)
@click.option('--clean', '--clear', '--prune', 'clean_first', is_flag=True,
              help='Remove build artifacts before building')
@click.option('-v', '--verbose', help="Be more verbose", is_flag=True)
def build(package=None, **options):
    """Build the package and create a kecpkg file."""
    echo_info('Locating package ``'.format(package))
    package_dir = get_package_dir(package_name=package)
    package_name = os.path.basename(package_dir)
    echo_info('Package `{}` has been selected'.format(package_name))
    settings = load_settings(package_dir=package_dir)

    # ensure build directory is there
    build_dir = settings.get('build_dir', 'dist')
    build_path = os.path.join(package_dir, build_dir)

    if options.get('clean_first'):
        remove_path(build_path)
    ensure_dir_exists(build_path)

    # do package building
    build_package(package_dir, build_path, settings, verbose=options.get('verbose'))


def build_package(package_dir, build_path, settings, verbose=False):
    """Perform the actual building of the kecpkg zip."""
    additional_exclude_paths = settings.get('exclude_paths')

    artifacts = get_artifacts_on_disk(package_dir, verbose=verbose, additional_exclude_paths=additional_exclude_paths)
    dist_filename = '{}-{}-py{}.kecpkg'.format(settings.get('package_name'), settings.get('version'),
                                               settings.get('python_version'))
    echo_info('Creating package name `{}`'.format(dist_filename))

    with ZipFile(os.path.join(build_path, dist_filename), 'w') as dist_zip:
        for artifact in artifacts:
            dist_zip.write(os.path.join(package_dir, artifact), arcname=artifact)



from __future__ import print_function

import os
import subprocess
import sys

import six

from kecpkg.commands.utils import echo_failure, echo_info, echo_success
from kecpkg.files.rendering import render_to_file
from kecpkg.utils import ensure_dir_exists, get_proper_python, NEED_SUBPROCESS_SHELL, venv


def create_package(package_dir, settings):
    """
    Create the package directory.

    package_name  (or package_dir)
    +-- README.md
    +-- requirements.txt
    +-- package_info.json
    +-- main.py  (settable with settings['entrypoint_script']

    :param package_dir: the full path to the package dir
    :param settings: settings dict
    """
    ensure_dir_exists(package_dir)
    render_to_file('README.md', content=settings, target_dir=package_dir)
    render_to_file('requirements.txt', content=settings, target_dir=package_dir)
    render_to_file('package_info.json', content=dict(requirements_txt='requirements.txt',
                                                     entrypoint_script=settings.get('entrypoint_script'),
                                                     entrypoint_func=settings.get('entrypoint_func')),
                   target_dir=package_dir)
    render_to_file('.gitignore', content=dict(), target_dir=package_dir)
    render_to_file('.env', content=dict(), target_dir=package_dir)

    # runconfigurations
    run_configurations_path = os.path.join(package_dir, '.idea', 'runConfigurations')
    ensure_dir_exists(run_configurations_path)
    render_to_file('Upload_the_kecpkg.xml', content=dict(), target_dir=run_configurations_path)
    render_to_file('Build_the_kecpkg.xml', content=dict(), target_dir=run_configurations_path)

    script_filename = '{}.py'.format(settings.get('entrypoint_script'))

    render_to_file(script_filename, content=settings, template='script.py.template', target_dir=package_dir)


def create_venv(package_dir, settings, pypath=None, use_global=False, verbose=False):
    """
    Create the virtual environment in `venv` for the package.

    The virtual environment path name can be set in the settings.

    package_dir
    +-- venv  (the virtual environment based on the choosen python version)
        +-- ...

    :param package_dir: the full path to the package directory
    :param settings: the settings dict (including the venv_dir name to create the right venv)
    """
    venv_dir = os.path.join(package_dir, settings.get('venv_dir'))

    if not pypath:
        from distutils.spawn import find_executable
        pypath = find_executable(get_proper_python())

    command = [sys.executable, '-m', 'virtualenv', venv_dir, '-p', pypath]
    if use_global:  # no cov
        command.append('--system-site-packages')
    if not verbose:  # no cov
        command.append('-qqq')
    if six.PY3:
        result = subprocess.run(command, shell=NEED_SUBPROCESS_SHELL)
        return result.returncode
    elif six.PY2:
        result = subprocess.check_output(command, shell=NEED_SUBPROCESS_SHELL)
        return result and 0 or -1


def pip_install_venv(package_dir, settings, verbose=False):
    """
    Install requirements into the virtual environment.

    :param package_dir: the full path to the package directory
    :param settings: the settings dict (incluing the venv_dir name)
    :param verbose: (optional) be more verbose if set to True, defaults to False
    """
    venv_dir = os.path.join(package_dir, settings.get('venv_dir'))
    if not os.path.exists(venv_dir):
        echo_failure('virtual environment directory `{}` does not exists, nothing to install'.format(venv_dir))
        sys.exit(1)

    if not os.path.exists(os.path.join(package_dir, settings.get('requirements_filename'))):
        echo_failure('could not find requirements.txt to install, check if `{}` exists or update settings'.format(
            settings.get('requirements_filename')))
        sys.exit(1)

    install_command = [sys.executable, '-m', 'pip', 'install', '-r',
                       os.path.join(package_dir, settings.get('requirements_filename'))]

    if not verbose:  # no cov
        install_command.append('-qqq')

    with venv(venv_dir):
        echo_info('Installing requirements from `{}` into the virtual environment `{}`'.
                  format(settings.get('requirements_filename'), settings.get('venv_dir')))
        result = None
        if six.PY3:
            result = subprocess.run(install_command, shell=NEED_SUBPROCESS_SHELL)
            return result.returncode
        elif six.PY2:
            result = subprocess.check_output(install_command, shell=NEED_SUBPROCESS_SHELL)
            return result and 0 or -1

    if result:
        echo_success(str(result))

    return result.returncode



import os

import pytest
from click.testing import CliRunner
from envparse import Env

from kecpkg.cli import kecpkg
from kecpkg.utils import get_package_dir
from tests.utils import temp_chdir, BaseTestCase


@pytest.mark.skipif("os.getenv('TRAVIS', False)",
                    reason="Skipping tests when using Travis, as upload of services cannot be testing securely")
class TestCommandUpload(BaseTestCase):
    def test_upload_non_interactive(self):
        pkgname = 'new_pkg'
        env = Env.read_envfile()

        with temp_chdir() as d:
            runner = CliRunner()
            result = runner.invoke(kecpkg, ['new', pkgname, '--no-venv'])
            package_dir = get_package_dir(pkgname)
            self.assertTrue(os.path.exists(package_dir))

            os.chdir(package_dir)

            result = runner.invoke(kecpkg, ['build', pkgname])
            self.assertEqual(result.exit_code, 0)
            self.assertExists(os.path.join(package_dir, 'dist'))
            pkgpath = os.path.join(package_dir, 'dist', '{}-0.0.1-py3.5.kecpkg'.format(pkgname))
            self.assertExists(pkgpath)

            result = runner.invoke(kecpkg, [
                'upload', pkgname,
                '--url', os.environ.get('KECHAIN_URL'),
                '--token', os.environ.get('KECHAIN_TOKEN'),
                '--scope-id', os.environ.get('KECHAIN_SCOPE_ID'),
                '--kecpkg', pkgpath,
                '--store'  # store the service_id in the settings (for teardown)
            ])
            self.assertEqual(result.exit_code, 0)

            # teardown the just uploaded service
            from kecpkg.settings import load_settings
            settings = load_settings(package_dir=get_package_dir(pkgname))

            from pykechain import get_project
            scope = get_project(
                url=os.environ.get('KECHAIN_URL'),
                token=os.environ.get('KECHAIN_TOKEN'),
                scope_id=os.environ.get('KECHAIN_SCOPE_ID')
            )
            service = scope.service(pk=settings.get('service_id'))
            service.delete()



from io import open

import os
from setuptools import find_packages, setup

PACKAGE_NAME = 'kecpkg'
HERE = os.path.abspath(os.path.dirname(__file__))

with open(os.path.join(HERE, 'README.rst'), 'r') as f:
    long_description = f.read()

ABOUT = {}
with open(os.path.join(PACKAGE_NAME, '__init__.py'), 'r') as f:
    exec(f.read(), ABOUT)

setup(
    name='kecpkg-tools',
    version=ABOUT.get('__version__'),
    description='',
    long_description=long_description,
    author='Jochem Berends',
    author_email='jochem.berends@ke-works.com',
    maintainer='Jochem Berends',
    maintainer_email='jochem.berends@ke-works.com',
    url='https://github.com/jberends/kecpkg-tools',
    license='Apache-2.0',

    keywords=(
        'python',
        'package tools',
        'pykechain',
        'KE-chain',
        'Services Integration Module',
        'SIM'
    ),

    classifiers=(
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
    ),

    install_requires=(
        'click',
        'atomicwrites',
        'jinja2',
        'pykechain>=1.13',
    ),

    tests_require=(
        'coverage',
        'pytest',
        'flake8',
        'virtualenv'
    ),

    packages=find_packages(exclude=['tests']),

    # to include the templates in the bdist wheel, we need to add package_data here
    package_data={
        'kecpkg': ['files/templates/*.template',
                   'files/templates/.*.template']
    },

    entry_points={
        'console_scripts': (
            'kecpkg = kecpkg.cli:kecpkg',
        ),
    }
)



import os
import sys

import click

from kecpkg.commands.utils import echo_failure, CONTEXT_SETTINGS, echo_info, echo_success
from kecpkg.create import create_package, create_venv, pip_install_venv
from kecpkg.settings import load_settings, copy_default_settings, save_settings
from kecpkg.utils import normalise_name


@click.command(short_help="Create a new kecpkg SIM script package",
               context_settings=CONTEXT_SETTINGS)
@click.argument('package', required=False)
@click.option('--venv', help="name of the virtual python environment to create")
@click.option('--script', help="name of the script inside the package that contains the entrypoint")
@click.option('--global-packages', is_flag=True,
              help='Gives created virtual envs access to the global site-packages.')
@click.option('--no-venv', help="suppress the creation of the virtual environment", is_flag=True)
@click.option('-v', '--verbose', help="Be more verbose", is_flag=True)
def new(package=None, **options):
    """
    Create a new package directory structure.

    <pkg dir>
    +-- venv
    |   +-- ... <the virtualenvironment>
    +-- README.md
    +-- requirements.txt
    +-- script.py
    +-- package-info.json
    +-- .gitignore
    +-- .kecpkg-settings.json
    """
    settings = load_settings(lazy=True)
    if not settings:
        settings = copy_default_settings()
    package_root_dir = os.getcwd()

    # set the package name, clean an normalise using snake_case
    package_name = package or click.prompt("Package name")
    package_name = normalise_name(package_name)

    # save to settings
    settings['package_name'] = package_name

    package_dir = os.path.join(package_root_dir, package_name)
    if os.path.exists(package_dir):
        echo_failure("Directory '{}' already exists.".format(package_dir))
        sys.exit(1)

    if not package:
        settings['version'] = click.prompt('Version', default=settings.get('version', '0.0.1'))
        settings['description'] = click.prompt('Description', default='')
        settings['name'] = click.prompt('Author', default=settings.get('name', os.environ.get('USER', '')))
        settings['email'] = click.prompt('Author\'s email', default=settings.get('email', ''))
        settings['python_version'] = click.prompt('Python version (choose from: {})'.format(settings.get('pyversions')),
                                                  default='3.5')
    if options.get('script'):
        script_base = normalise_name(options.get('script').replace('.py', ''))
        echo_info('Setting the script to `{}`'.format(script_base))
        settings['entrypoint_script'] = script_base

    if options.get('venv'):
        settings['venv_dir'] = normalise_name(options.get('venv'))

    echo_info("Creating package structure")
    create_package(package_dir, settings=settings)
    if not options.get('no_venv'):
        echo_info("Creating virtual environment")
        create_venv(package_dir, settings, pypath=None, use_global=options.get('global_packages'),
                    verbose=options.get('verbose'))
        pip_install_venv(package_dir, settings, verbose=options.get('verbose'))
    else:
        settings['venv_dir'] = None

    # save the settings (in the package_dir)
    save_settings(settings, package_dir=package_dir)

    echo_success('Package `{package_name}` created in `{package_dir}`'.format(package_name=package_name,
                                                                              package_dir=package_dir))



import os

import click

from kecpkg.commands.utils import CONTEXT_SETTINGS, echo_failure, echo_warning
from kecpkg.settings import load_settings
from kecpkg.utils import get_package_name, get_package_dir, remove_path


@click.command(context_settings=CONTEXT_SETTINGS,
               short_help="Removes a project's build artifacts")
@click.argument('package', required=False)
@click.option('--force', '-f', is_flag=True, help="Forcefully removes the project build artifacts")
def prune(package, **options):
    """Remove a project's build artifacts."""
    package_name = package or get_package_name() or click.prompt('Provide package name')
    package_dir = get_package_dir(package_name)

    settings = load_settings(package_name)
    # ensure build directory is there
    build_dir = settings.get('build_dir', 'dist')
    build_path = os.path.join(package_dir, build_dir)

    if os.path.exists(build_path):
        if options.get('force') or click.confirm(
                "Do you want to prune build artifacts for package '{}'?".format(package_name)):
            remove_path(build_path)
            if os.path.exists(build_path):
                echo_failure('Something went wrong pruning pacakage `{}`'.format(package_name))
        else:
            echo_warning('Package `{}` will not be pruned'.format(package_name))
    else:
        echo_failure('Package `{}` does not exist'.format(package_name))



import click

CONTEXT_SETTINGS = {
    'help_option_names': ['-h', '--help'],
}
UNKNOWN_OPTIONS = {
    'ignore_unknown_options': True,
}.update(CONTEXT_SETTINGS)


def echo_success(text, nl=True):
    """
    Write to the console as a success (Cyan bold).

    :param text: string to write
    :param nl: add newline
    """
    click.secho(text, fg='cyan', bold=True, nl=nl)


def echo_failure(text, nl=True):
    """
    Write to the console as a failure (Red bold).

    :param text: string to write
    :param nl: add newline
    """
    click.secho(text, fg='red', bold=True, nl=nl)


def echo_warning(text, nl=True):
    """
    Write to the console as a warning (Yellow bold).

    :param text: string to write
    :param nl: add newline
    """
    click.secho(text, fg='yellow', bold=True, nl=nl)


def echo_waiting(text, nl=True):
    """
    Write to the console as a waiting (Magenta bold).

    :param text: string to write
    :param nl: add newline
    """
    click.secho(text, fg='magenta', bold=True, nl=nl)


def echo_info(text, nl=True):
    """
    Write to the console as a informational (bold).

    :param text: string to write
    :param nl: add newline
    """
    click.secho(text, bold=True, nl=nl)



import os

from click.testing import CliRunner

from kecpkg.cli import kecpkg
from tests.utils import temp_chdir, BaseTestCase


class TestCommandNew(BaseTestCase):
    def test_new_non_interactive_no_venv(self):
        pkgname = 'new_pkg'

        with temp_chdir() as d:
            runner = CliRunner()
            result = runner.invoke(kecpkg, ['new', pkgname, '--no-venv'])

            self.assertEqual(result.exit_code, 0)

            self.assertTrue(os.path.exists(os.path.join(d, pkgname)))
            self.assertTrue(os.path.exists(os.path.join(d, pkgname, 'script.py')))
            self.assertTrue(os.path.exists(os.path.join(d, pkgname, 'package_info.json')))
            self.assertTrue(os.path.exists(os.path.join(d, pkgname, 'README.md')))
            self.assertTrue(os.path.exists(os.path.join(d, pkgname, 'requirements.txt')))
            self.assertTrue(os.path.exists(os.path.join(d, pkgname, '.gitignore')))
            self.assertTrue(os.path.exists(os.path.join(d, pkgname, '.kecpkg_settings.json')))
            self.assertTrue(os.path.exists(os.path.join(d, pkgname, '.idea')))
            self.assertTrue(os.path.exists(os.path.join(d, pkgname, '.env')))
            self.assertTrue(os.path.exists(os.path.join(d, pkgname, '.idea', 'runConfigurations')))
            self.assertTrue(
                os.path.exists(os.path.join(d, pkgname, '.idea', 'runConfigurations', 'Build_the_kecpkg.xml')))
            self.assertTrue(
                os.path.exists(os.path.join(d, pkgname, '.idea', 'runConfigurations', 'Upload_the_kecpkg.xml')))

            self.assertFalse(os.path.exists(os.path.join(d, pkgname, 'venv')))

    def test_new_non_interactive_with_venv(self):
        pkgname = 'new_pkg'

        with temp_chdir() as d:
            runner = CliRunner()
            result = runner.invoke(kecpkg, ['new', pkgname])

            e = result.exception

            self.assertEqual(result.exit_code, 0)

            self.assertTrue(os.path.exists(os.path.join(d, pkgname, 'venv')))
            self.assertTrue(os.path.exists(os.path.join(d, pkgname, 'venv', 'bin', 'activate')))

    def test_new_non_iteractive_with_alternate_script(self):
        pkgname = 'new_pkg'

        with temp_chdir() as d:
            runner = CliRunner()
            result = runner.invoke(kecpkg, ['new', pkgname, '--script', 'foobar', '--no-venv'])

            self.assertEqual(result.exit_code, 0)

            self.assertTrue(os.path.exists(os.path.join(d, pkgname, 'foobar.py')))

    def test_new_non_interactive_with_alternate_venv_name(self):
        pkgname = 'new_pkg'
        venv_name = '_v'

        with temp_chdir() as d:
            runner = CliRunner()
            result = runner.invoke(kecpkg, ['new', pkgname, '--venv', venv_name])

            self.assertEqual(result.exit_code, 0)

            self.assertTrue(os.path.exists(os.path.join(d, pkgname, venv_name)))
            self.assertTrue(os.path.exists(os.path.join(d, pkgname, venv_name, 'bin', 'activate')))



import os
from zipfile import ZipFile

from click.testing import CliRunner

from kecpkg.cli import kecpkg
from kecpkg.settings import load_settings, save_settings, copy_default_settings
from kecpkg.utils import get_package_dir, ensure_dir_exists
from tests.utils import temp_chdir, BaseTestCase, touch_file


class TestCommandPurge(BaseTestCase):
    def test_build_non_interactive(self):
        pkgname = 'new_pkg'

        with temp_chdir() as d:
            runner = CliRunner()
            result = runner.invoke(kecpkg, ['new', pkgname, '--no-venv'])
            package_dir = get_package_dir(pkgname)
            self.assertTrue(os.path.exists(package_dir))

            os.chdir(package_dir)

            result = runner.invoke(kecpkg, ['build', pkgname])
            self.assertEqual(result.exit_code, 0)
            self.assertExists(os.path.join(package_dir, 'dist'))

            # check if dist is filled
            package_dir_contents = os.listdir(os.path.join(package_dir, 'dist'))
            self.assertTrue(len(package_dir_contents), 1)

    def test_build_with_prune(self):
        pkgname = 'new_pkg'

        with temp_chdir() as d:
            runner = CliRunner()
            result = runner.invoke(kecpkg, ['new', pkgname, '--no-venv'])
            package_dir = get_package_dir(pkgname)
            self.assertTrue(os.path.exists(package_dir))

            os.chdir(package_dir)

            result = runner.invoke(kecpkg, ['build', pkgname])
            self.assertEqual(result.exit_code, 0)
            self.assertExists(os.path.join(package_dir, 'dist'))

            # check if dist is filled
            package_dir_contents = os.listdir(os.path.join(package_dir, 'dist'))
            self.assertTrue(len(package_dir_contents), 1)

            # restart the build, with prune and check if dist still has 1
            result = runner.invoke(kecpkg, ['build', pkgname, '--prune'])
            self.assertEqual(result.exit_code, 0)
            self.assertExists(os.path.join(package_dir, 'dist'))

            # check if dist is filled
            package_dir_contents = os.listdir(os.path.join(package_dir, 'dist'))
            self.assertTrue(len(package_dir_contents), 1)



    def test_build_with_extra_ignores(self):
        pkgname = 'new_pkg'

        with temp_chdir() as d:
            runner = CliRunner()
            result = runner.invoke(kecpkg, ['new', pkgname, '--no-venv'])
            package_dir = get_package_dir(pkgname)
            self.assertTrue(os.path.exists(package_dir))
            ensure_dir_exists(os.path.join(package_dir, 'data'))

            # add additional files (to exclude for building later)
            touch_file(os.path.join(package_dir, 'data','somefile.txt'))
            touch_file(os.path.join(package_dir, 'local_extra_file.someext.txt'))

            os.chdir(package_dir)

            # check if those files exists
            package_dir_contents = os.listdir(os.path.join(package_dir))
            self.assertTrue('local_extra_file.someext.txt' in package_dir_contents)
            self.assertTrue('data' in package_dir_contents)

            # set exclude_paths in settings
            settings = copy_default_settings()
            settings["exclude_paths"] = ["data", "local_extra_file.*"]
            save_settings(settings, package_dir=package_dir)

            # run the builder
            result = runner.invoke(kecpkg, ['build', pkgname, '--verbose'])
            self.assertEqual(result.exit_code, 0)
            self.assertExists(os.path.join(package_dir, 'dist'))

            # check the zip such that the extra files are not packaged
            dist_list = os.listdir(os.path.join(package_dir, 'dist'))
            zipfile = ZipFile(os.path.join(package_dir, 'dist', dist_list[0]), 'r')
            contents = zipfile.namelist()

            self.assertFalse('local_extra_file.someext.txt' in contents)
            self.assertFalse('data' in contents)